import wx #graphic lib
import Image #requires PIL
import os
import fnmatch
import shutil
import signal
import threading
import time # for threading
import sys

#sys.setrecursionlimit(9500)

import StoboEngine
from StoboCommon import *

MAIN_W = 700
MAIN_H = 600
MIN_W = 500
MIN_H = 500

ID_ABOUT = 101
ID_USAGE = 103
ID_EXIT  = 102

# preview for huge images is turned off during processing
FSIZE = 4700000 #bytes

class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        self.proc_state = 0
        self.Core = StoboEngine.Engine() #engine used by gui
        self.timer_loop = "off"
        self.queueTimer = None
        self.create_log = 0

        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)    
        
        this_path = os.path.dirname(sys.argv[0])
        full_path = os.path.abspath(this_path)
        icon_path = os.path.join(full_path, 'logo.ico')      
        self.SetIcon(wx.Icon(icon_path, wx.BITMAP_TYPE_ICO));
        
        
        self.notebook_1 = wx.Notebook(self, -1, style=0)
        #'image' tab
        self.panel_3 = wx.Panel(self.notebook_1, -1)
        #'global' tab
        self.notebook_1_pane_1 = wx.Panel(self.notebook_1, -1)
        self.sizer_5_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, "output directory")
        self.sizer_30_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, "custom size")
        self.sizer_16_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, "transform queue")
        self.sizer_27_staticbox = wx.StaticBox(self.panel_3, -1, "target image")
        self.sizer_3_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, "processing directory")
        
        # Menu Bar
        self.MainFrame_menubar = wx.MenuBar()
        self.SetMenuBar(self.MainFrame_menubar)
        wx.glade_main_menu = wx.Menu()
        wx.glade_main_menu.Append(ID_EXIT, "&Quit\tCtrl+q", "", wx.ITEM_NORMAL)
        #wxglade_tmp_menu.Append(wx.NewId(), "&Quit\tCtrl+q", "", wx.ITEM_NORMAL)
        self.MainFrame_menubar.Append(wx.glade_main_menu, "&File")
        
        
        wx.glade_help_menu = wx.Menu()
        wx.glade_help_menu.Append(ID_ABOUT, "&About\tCtrl+a", "", wx.ITEM_NORMAL)
        wx.glade_help_menu.Append(ID_USAGE, "&Usage\tCtrl+u", "", wx.ITEM_NORMAL)
        self.MainFrame_menubar.Append(wx.glade_help_menu, "&Help")
        # Menu Bar end
        
        
        self.MainFrame_statusbar = self.CreateStatusBar(1, wx.ST_SIZEGRIP)#sizegrip styles the grip to btm_rt
        
        self.btn_in = wx.Button(self.notebook_1_pane_1, -1, "directory:")
        self.textctrl_in = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.text_list_in = wx.TextCtrl(self.notebook_1_pane_1, -1, "", style=wx.TE_MULTILINE|wx.TE_READONLY)
        self.btn_out = wx.Button(self.notebook_1_pane_1, -1, "directory:")
        self.textctrl_out = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.text_list_out = wx.TextCtrl(self.notebook_1_pane_1, -1, "", style=wx.TE_MULTILINE|wx.TE_READONLY)
        self.radio_btn_originals = wx.RadioButton(self.notebook_1_pane_1, -1, "")
        self.label_3_copy_1 = wx.StaticText(self.notebook_1_pane_1, -1, "no renaming")
        self.radio_btn_customs = wx.RadioButton(self.notebook_1_pane_1, -1, "")
        self.panel_4 = wx.Panel(self.notebook_1_pane_1, -1)
        self.text_ctrl_cust_name = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.label_3_copy_copy_copy_copy = wx.StaticText(self.notebook_1_pane_1, -1, " (name)")
        self.text_ctrl_cust_num = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.spin_ctrl_iter = wx.SpinCtrl(self.notebook_1_pane_1, -1, "1", min=1, max=100)
        self.label_3_copy_copy_copy = wx.StaticText(self.notebook_1_pane_1, -1, " (sequence)")
        self.list_box_queue = wx.ListBox(self.notebook_1_pane_1, -1, choices=[], style=wx.LB_SINGLE)
        self.lbl_format = wx.StaticText(self.notebook_1_pane_1, -1, "format: ")
        self.combo_box_format = wx.ComboBox(self.notebook_1_pane_1, -1, choices=["---", "png", "jpeg", "gif"], style=wx.CB_DROPDOWN|wx.CB_READONLY)
        self.lbl_rotate = wx.StaticText(self.notebook_1_pane_1, -1, "rotate: ")
        self.combo_box_rotate = wx.ComboBox(self.notebook_1_pane_1, -1, choices=["---", "90", "180", "270"], style=wx.CB_DROPDOWN|wx.CB_READONLY)
        self.lbl_size = wx.StaticText(self.notebook_1_pane_1, -1, "size: ")
        self.combo_box_size = wx.ComboBox(self.notebook_1_pane_1, -1, choices=["---", "thmb.", "medm.", "cust."], style=wx.CB_DROPDOWN|wx.CB_READONLY)
        self.label_1_copy_1_copy = wx.StaticText(self.notebook_1_pane_1, -1, "w:")
        self.spin_ctrl_width = wx.SpinCtrl(self.notebook_1_pane_1, -1, "", min=0, max=4000)
        self.label_1_copy_copy_copy = wx.StaticText(self.notebook_1_pane_1, -1, "h:")
        self.spin_ctrl_height = wx.SpinCtrl(self.notebook_1_pane_1, -1, "", min=0, max=4000)
        self.combo_box_measure = wx.ComboBox(self.notebook_1_pane_1, -1, choices=["px", "mm", "in"], style=wx.CB_DROPDOWN|wx.CB_READONLY)
        self.btnStart = wx.Button(self.notebook_1_pane_1, -1, "START")
        self.btnStop = wx.Button(self.notebook_1_pane_1, -1, "STOP")
        self.prvw_pnl = wx.Panel(self.panel_3, -1)
        #self.prvw_img = wx.StaticBitmap(self.panel_3, -1)
        self.panel_1 = wx.Panel(self.panel_3, -1)
        self.img_bck_btn = wx.Button(self.panel_3, -1, "<")
        self.img_stp_btn = wx.Button(self.panel_3, -1, "||")
        self.img_fwd_btn = wx.Button(self.panel_3, -1, ">")
        self.panel_2 = wx.Panel(self.panel_3, -1)

        self.__set_properties()
        self.__do_layout()
        
        #self.Bind(wx.EVT_MENU, self.QuitOn, id=-1)
        self.Bind(wx.EVT_BUTTON, self.OpenProcessingFile, self.btn_in)
        self.Bind(wx.EVT_BUTTON, self.OpenOutputFile, self.btn_out)
        self.Bind(wx.EVT_RADIOBUTTON, self.og_radio, self.radio_btn_originals)
        self.Bind(wx.EVT_RADIOBUTTON, self.cust_radio, self.radio_btn_customs)
        self.Bind(wx.EVT_TEXT, self.name_update, self.text_ctrl_cust_name)
        self.Bind(wx.EVT_TEXT, self.sequence_update, self.text_ctrl_cust_num)
        self.Bind(wx.EVT_SPINCTRL, self.spin_iter, self.spin_ctrl_iter)
        self.Bind(wx.EVT_COMBOBOX, self.combo_format, self.combo_box_format)
        self.Bind(wx.EVT_COMBOBOX, self.combo_rotate, self.combo_box_rotate)
        self.Bind(wx.EVT_COMBOBOX, self.combo_size, self.combo_box_size)
        self.Bind(wx.EVT_BUTTON, self.start_proc, self.btnStart)
        self.Bind(wx.EVT_BUTTON, self.stop_proc, self.btnStop)
        self.Bind(wx.EVT_BUTTON, self.img_bck, self.img_bck_btn)
        self.Bind(wx.EVT_BUTTON, self.img_stp, self.img_stp_btn)
        self.Bind(wx.EVT_BUTTON, self.img_fwd, self.img_fwd_btn)
        self.Bind(wx.EVT_LISTBOX, self.list_box, self.list_box_queue)
        self.Bind(wx.EVT_SPINCTRL, self.spin_width, self.spin_ctrl_width)
        self.Bind(wx.EVT_SPINCTRL, self.spin_height, self.spin_ctrl_height)
        self.Bind(wx.EVT_COMBOBOX, self.combo_measure, self.combo_box_measure)
        # end wxGlade
        
        self.Bind(wx.EVT_MENU, self.QuitOn,  id=ID_EXIT)
        self.Bind(wx.EVT_MENU, self.AboutOn, id=ID_ABOUT)
        self.Bind(wx.EVT_MENU, self.UsageOn, id=ID_USAGE)
        
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_CLOSE, self.QuitWindow)
        # end wxGlade
    
    def queue_timer_fast(self):
        self.queueTimer = wx.Timer(self, id=1)
        self.queueTimer.Start(800) #was 300
        self.Bind(wx.EVT_TIMER, self.start_proc, id=1)
        
    def queue_timer_slow(self):
        self.queueTimer = wx.Timer(self, id=1)
        self.queueTimer.Start(4000)
        self.Bind(wx.EVT_TIMER, self.start_proc, id=1)
        
    def queue_timer_stop(self):
        self.queueTimer = wx.Timer(self, id=1)
        if self.queueTimer:
            
            self.queueTimer.Stop()
            del self.queueTimer

    def __set_properties(self):
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle("Stobo")
        self.SetSize((MAIN_W, MAIN_H))
        self.SetMinSize((MIN_W, MIN_H))
        #self.SetMinSize(self.v_sizer.GetMinSize())
        self.MainFrame_statusbar.SetStatusWidths([-1])
        # statusbar fields
        MainFrame_statusbar_fields = ["- progress bar --"]
        for i in range(len(MainFrame_statusbar_fields)):
            self.MainFrame_statusbar.SetStatusText(MainFrame_statusbar_fields[i], i)
        self.text_list_in.SetDefaultStyle(wx.TextAttr(wx.NullColour)) #rem hglt
        self.text_list_out.SetDefaultStyle(wx.TextAttr(wx.NullColour))#rem hglt
        self.text_list_in.Enable(False)
        self.text_list_out.Enable(False)
        self.radio_btn_originals.SetMinSize((-1, -1))
        self.radio_btn_originals.SetFocus()
        self.radio_btn_customs.SetMinSize((-1,-1))
        self.text_ctrl_cust_name.SetMinSize((-1,-1))
        self.text_ctrl_cust_num.SetMinSize((-1,-1))
        self.spin_ctrl_iter.SetMinSize((40, -1))
        self.combo_box_format.SetMinSize((85, -1))
        self.combo_box_format.SetSelection(0)
        self.combo_box_rotate.SetMinSize((85, -1))
        self.combo_box_rotate.SetSelection(0)
        self.combo_box_size.SetMinSize((85, -1))
        self.combo_box_size.SetSelection(0)
        self.combo_box_measure.SetSelection(0)
        self.img_bck_btn.SetMinSize((75, -1))
        self.img_stp_btn.SetMinSize((60, -1))
        self.img_fwd_btn.SetMinSize((75, -1))
        # end wxGlade
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: MyFrame.__do_layout
        self.prvw_szr_h = wx.BoxSizer(wx.HORIZONTAL)
        self.prvw_szr_v = wx.BoxSizer(wx.VERTICAL)
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_9 = wx.BoxSizer(wx.VERTICAL)
        sizer_13_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_32 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_20 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_26 = wx.BoxSizer(wx.VERTICAL)
        sizer_27 = wx.StaticBoxSizer(self.sizer_27_staticbox, wx.VERTICAL)
        sizer_11 = wx.BoxSizer(wx.VERTICAL)
        sizer_13 = wx.BoxSizer(wx.VERTICAL)
        sizer_14 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_15 = wx.BoxSizer(wx.VERTICAL)
        sizer_16 = wx.StaticBoxSizer(self.sizer_16_staticbox, wx.HORIZONTAL)
        sizer_17 = wx.BoxSizer(wx.VERTICAL)
        sizer_30 = wx.StaticBoxSizer(self.sizer_30_staticbox, wx.HORIZONTAL)
        sizer_10 = wx.BoxSizer(wx.VERTICAL)
        sizer_12 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_10_copy_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_19 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_21 = wx.BoxSizer(wx.VERTICAL)
        sizer_8_copy_copy_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_9_copy_copy_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_23 = wx.BoxSizer(wx.VERTICAL)
        sizer_8_copy_1_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_9_copy_1_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_23_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_28 = wx.BoxSizer(wx.VERTICAL)
        sizer_29_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_34_copy_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_34_copy_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_37 = wx.BoxSizer(wx.VERTICAL)
        sizer_29_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_34_copy_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_34_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_37_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_29 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_34_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_34 = wx.BoxSizer(wx.VERTICAL)
        sizer_39 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_25 = wx.BoxSizer(wx.VERTICAL)
        sizer_8 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_31 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_35 = wx.BoxSizer(wx.VERTICAL)
        sizer_38 = wx.BoxSizer(wx.VERTICAL)
        sizer_40 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_18_copy_1_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_35_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_38_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_40_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_18_copy_1_copy_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_33 = wx.BoxSizer(wx.VERTICAL)
        sizer_18_copy_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_18 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_22 = wx.BoxSizer(wx.VERTICAL)
        sizer_24 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_9_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_22_copy_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_24_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_22_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_18_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_5 = wx.StaticBoxSizer(self.sizer_5_staticbox, wx.VERTICAL)
        sizer_7 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3 = wx.StaticBoxSizer(self.sizer_3_staticbox, wx.VERTICAL)
        sizer_6 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_4.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_2.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_3.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_6.Add(self.btn_in, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_6.Add(self.textctrl_in, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_3.Add(sizer_6, 0, wx.EXPAND, 0)
        sizer_3.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_3.Add(self.text_list_in, 1, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_2.Add(sizer_3, 2, wx.EXPAND, 0)
        sizer_2.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_5.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_7.Add(self.btn_out, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_7.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_7.Add(self.textctrl_out, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_5.Add(sizer_7, 0, wx.EXPAND, 0)
        sizer_5.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_5.Add(self.text_list_out, 1, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_2.Add(sizer_5, 2, wx.EXPAND, 0)
        sizer_2.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_4.Add(sizer_2, 1, wx.EXPAND, 0)
        sizer_4.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_15.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_17.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_18_copy.Add(self.radio_btn_originals, 0, wx.ALIGN_CENTER_VERTICAL|wx.SHAPED, 0)
        sizer_22_copy.Add(sizer_18_copy, 1, 0, 0)
        sizer_9_copy.Add(sizer_22_copy, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_24_copy.Add((7, 20), 0, wx.EXPAND, 4)
        sizer_24_copy.Add(self.label_3_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_22_copy_copy.Add(sizer_24_copy, 1, wx.EXPAND, 0)
        sizer_9_copy.Add(sizer_22_copy_copy, 1, 0, 0)
        sizer_24.Add(sizer_9_copy, 0, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_22.Add(sizer_24, 0, wx.EXPAND, 0)
        sizer_22.Add((20, 6), 0, wx.EXPAND, 0)
        sizer_25.Add(sizer_22, 1, wx.EXPAND, 0)
        sizer_18.Add(self.radio_btn_customs, 1, wx.EXPAND, 0)
        sizer_33.Add(sizer_18, 1, wx.EXPAND, 0)
        sizer_18_copy_2.Add(self.panel_4, 1, wx.EXPAND, 0)
        sizer_33.Add(sizer_18_copy_2, 1, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_31.Add(sizer_33, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_38_copy.Add((20, 2), 0, 0, 0)
        sizer_18_copy_1_copy_copy.Add(self.text_ctrl_cust_name, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40_copy.Add(sizer_18_copy_1_copy_copy, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40_copy.Add((1, 20), 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_38_copy.Add(sizer_40_copy, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_38_copy.Add((20, 2), 0, 0, 0)
        sizer_35_copy.Add(sizer_38_copy, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_35_copy.Add(self.label_3_copy_copy_copy_copy, 1, wx.EXPAND, 0)
        sizer_31.Add(sizer_35_copy, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_31.Add((8, 20), 0, wx.EXPAND, 4)
        sizer_38.Add((20, 2), 0, 0, 0)
        sizer_18_copy_1_copy.Add(self.text_ctrl_cust_num, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40.Add(sizer_18_copy_1_copy, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40.Add(self.spin_ctrl_iter, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_38.Add(sizer_40, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_38.Add((20, 2), 0, 0, 0)
        sizer_35.Add(sizer_38, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_35.Add(self.label_3_copy_copy_copy, 1, wx.EXPAND, 0)
        sizer_31.Add(sizer_35, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_8.Add(sizer_31, 1, wx.EXPAND, 0)
        sizer_25.Add(sizer_8, 0, wx.EXPAND, 0)
        sizer_17.Add(sizer_25, 0, wx.EXPAND, 0)
        sizer_17.Add((20, 2), 0, wx.EXPAND, 0)
        sizer_39.Add(self.list_box_queue, 1, wx.EXPAND, 0)
        sizer_17.Add(sizer_39, 1, wx.EXPAND, 0)
        sizer_17.Add((20, 2), 0, wx.EXPAND, 0)
        sizer_34.Add(self.lbl_format, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_29.Add(sizer_34, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_34_copy.Add(self.combo_box_format, 0, 0, 0)
        sizer_29.Add(sizer_34_copy, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_28.Add(sizer_29, 1, wx.EXPAND, 0)
        sizer_37_copy.Add((20, 5), 0, wx.EXPAND, 0)
        sizer_28.Add(sizer_37_copy, 0, wx.EXPAND, 0)
        sizer_34_copy_1.Add(self.lbl_rotate, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_29_copy.Add(sizer_34_copy_1, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_34_copy_copy.Add(self.combo_box_rotate, 0, 0, 0)
        sizer_29_copy.Add(sizer_34_copy_copy, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_28.Add(sizer_29_copy, 1, wx.EXPAND, 0)
        sizer_37.Add((20, 5), 0, wx.EXPAND, 0)
        sizer_28.Add(sizer_37, 0, wx.EXPAND, 0)
        sizer_34_copy_2.Add(self.lbl_size, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_29_copy_1.Add(sizer_34_copy_2, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_34_copy_copy_1.Add(self.combo_box_size, 0, 0, 0)
        sizer_29_copy_1.Add(sizer_34_copy_copy_1, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_28.Add(sizer_29_copy_1, 1, wx.EXPAND, 0)
        sizer_28.Add((20, 1), 0, 0, 0)
        sizer_17.Add(sizer_28, 0, wx.EXPAND, 0)
        sizer_17.Add((20, 2), 0, 0, 0)
        sizer_17.Add((20, 2), 0, 0, 0)
        sizer_10.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_9_copy_1_copy.Add(self.label_1_copy_1_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_23_copy.Add(self.spin_ctrl_width, 0, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_9_copy_1_copy.Add(sizer_23_copy, 1, wx.ALIGN_BOTTOM, 0)
        sizer_8_copy_1_copy.Add(sizer_9_copy_1_copy, 1, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_12.Add(sizer_8_copy_1_copy, 1, wx.ALIGN_BOTTOM, 0)
        sizer_12.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_9_copy_copy_copy.Add(self.label_1_copy_copy_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_23.Add(self.spin_ctrl_height, 0, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_9_copy_copy_copy.Add(sizer_23, 1, wx.ALIGN_BOTTOM, 0)
        sizer_8_copy_copy_copy.Add(sizer_9_copy_copy_copy, 1, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_12.Add(sizer_8_copy_copy_copy, 1, wx.ALIGN_BOTTOM, 0)
        sizer_12.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_21.Add(self.combo_box_measure, 0, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_19.Add(sizer_21, 1, wx.ALIGN_BOTTOM, 0)
        sizer_10_copy_copy.Add(sizer_19, 1, wx.EXPAND|wx.ALIGN_BOTTOM, 0)
        sizer_12.Add(sizer_10_copy_copy, 1, wx.ALIGN_BOTTOM, 0)
        sizer_10.Add(sizer_12, 1, wx.EXPAND, 0)
        sizer_30.Add(sizer_10, 1, wx.EXPAND, 0)
        sizer_17.Add(sizer_30, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_16.Add(sizer_17, 1, wx.EXPAND, 0)
        sizer_15.Add(sizer_16, 1, wx.EXPAND, 0)
        sizer_15.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_4.Add(sizer_15, 1, wx.EXPAND, 0)
        sizer_4.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_11.Add(sizer_4, 1, wx.EXPAND, 0)
        sizer_11.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_14.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_14.Add(self.btnStart, 0, wx.ALIGN_RIGHT, 0)
        sizer_14.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_14.Add(self.btnStop, 0, wx.ALIGN_RIGHT, 0)
        sizer_14.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_13.Add(sizer_14, 1, wx.EXPAND, 0)
        sizer_11.Add(sizer_13, 0, wx.EXPAND, 0)
        sizer_11.Add((20, 10), 0, 0, 0)
        self.notebook_1_pane_1.SetSizer(sizer_11)
        sizer_20.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_26.Add((20, 10), 0, wx.EXPAND, 0)

        self.prvw_pnl.SetSizer(self.prvw_szr_v)
        self.prvw_szr_v.Add(self.prvw_szr_h, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND, 0) 
        self.prvw_img = wx.StaticBitmap(self.prvw_pnl, -1)
            
        self.prvw_szr_h.AddStretchSpacer()
        self.prvw_szr_h.Add(self.prvw_img, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        self.prvw_szr_h.AddStretchSpacer()
        
        sizer_27.Add(self.prvw_pnl, 1, wx.EXPAND, 0)
        
        bitmap = self.create_placeholder_image("- X -")
        self.prvw_img.SetBitmap(bitmap)

        sizer_26.Add(sizer_27, 1, wx.EXPAND, 0)
        sizer_26.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_20.Add(sizer_26, 1, wx.EXPAND, 0)
        sizer_20.Add((10, 20), 0, wx.EXPAND, 4)
        sizer_9.Add(sizer_20, 1, wx.EXPAND, 0)
        sizer_9.Add((20, 10), 0, wx.EXPAND, 0)
        sizer_32.Add(self.panel_1, 1, wx.EXPAND, 0)
        sizer_32.Add(self.img_bck_btn, 0, 0, 0)
        sizer_32.Add((10, 20), 0, 0, 0)
        sizer_32.Add(self.img_stp_btn, 0, 0, 0)
        sizer_32.Add((10, 20), 0, 0, 0)
        sizer_32.Add(self.img_fwd_btn, 0, 0, 0)
        sizer_32.Add(self.panel_2, 1, wx.EXPAND, 0)
        sizer_13_copy.Add(sizer_32, 1, wx.EXPAND, 0)
        sizer_9.Add(sizer_13_copy, 0, wx.EXPAND, 0)
        sizer_9.Add((20, 10), 0, 0, 0)
        self.panel_3.SetSizer(sizer_9)
        self.notebook_1.AddPage(self.notebook_1_pane_1, "Global")
        self.notebook_1.AddPage(self.panel_3, "Image")
        sizer_1.Add(self.notebook_1, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        self.Layout()
        #self.prvw_img.Center()
        # end wxGlade
        self.btnStart.Disable()
        self.btnStop.Disable()

    def create_placeholder_image(self, ptext):
        """
        creates a transparent rectangle
        places '- x -' near the center
        returns the image
        NOTE: does not work with in MSw, who-knows-why? not I.
        """
        try:
            font = wx.Font(15, wx.MODERN, wx.LIGHT, wx.LIGHT)
        except:
            font = wx.Font(15, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
            
        font.SetNoAntiAliasing()
        color = (0,0,255)#wx.BLACK
        pos = (120,10)
        text = ptext
        bitmap = wx.EmptyBitmap(300, 60)
              
        memory = wx.MemoryDC()    
        memory.SelectObject(bitmap)
       
        try:
            memory.SetPen(wx.Pen('#4c4c4c', 1, wx.TRANSPARENT))
            memory.DrawRectangle(0, 0, 300, 60)
        finally:
            memory.SelectObject(wx.NullBitmap)
        memory.Clear
        
        """
        text can never be placed over transparency in wxwidgets.
        a pixel is completely transparent or completely opaque
        """
 
        memory.SetFont(font)
        memory.SetTextForeground(color)
        memory.SelectObject(bitmap)
        try:
            memory.DrawText(text, pos[0],pos[1])
        finally:
            memory.SelectObject(wx.NullBitmap)
        memory.Clear
        
        timg = bitmap.ConvertToImage()
        if not timg.HasAlpha():
            timg.InitAlpha()
        for y in xrange(timg.GetHeight()):
            for x in xrange(timg.GetWidth()):
                pix = wx.Colour(timg.GetRed(x, y),
                                timg.GetGreen(x, y),
                                timg.GetBlue(x, y))
                if pix != wx.BLACK:
                    timg.SetAlpha(x, y, 0)

        bitmap = wx.BitmapFromImage(timg)
        #bitmap = timg.ConvertToBitmap() 
        return bitmap


    def QuitWindow(self, event):
        self.Core.go_thread = 0
        event.Skip()

    def QuitOn(self, event): # wxGlade: MyFrame.<event_handler>
        self.Core.go_thread = 0
        self.Close()
        event.Skip()

    def UsageOn(self, event): # wxGlade: MyFrame.<event_handler>
        dlg = wx.MessageDialog(self, 
           "1) Select a 'processing directory'.\n"
           "2) Select an 'output directory'.\n"
           "3) Specify output in the 'transform queue'\n"
           "4) Click 'START' to begin processing files.",
                              "Quick Start", wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def AboutOn(self, event): # wxGlade: MyFrame.<event_handler>
        self.info = wx.AboutDialogInfo()
        
        this_path = os.path.dirname(sys.argv[0])
        full_path = os.path.abspath(this_path)
        icon_path = os.path.join(full_path, 'stobo_logo_mid.ico')
        about_icon = wx.Icon(icon_path, wx.BITMAP_TYPE_ICO, 240, 240)
        self.info.SetIcon(about_icon)
        
        self.description = "Stobo modifies multiple image files.\n"
        self.description += "Its functionality includes: sequential renaming,\n"
        self.description += "copying, transforming and formatting of images.\n"
        #self.description += "\n"
        self.description += "Stobo is written by Bumblehead.\n" 
        self.info.SetName('Stobo')
        self.info.SetVersion('1.0')
        self.info.SetDescription(self.description)
        self.info.SetCopyright('(C) 2009 Bumblehead')
        self.info.SetWebSite('http://www.bumblehead.com')
        wx.AboutBox(self.info)

    def all_files(self, patterns='*', single_level=False, yield_folders=False):
        """
        recipe from the Python Cookbook,
        walk a tree and filter on extension
        dir_list = glob.glob('Pictures/scan/*.*') can't specify directory
        """
        # Expand patterns from semicilon-separated string to list
        patterns = patterns.split(';')
        for path, subdirs, files in os.walk(root):
            if yield_folders:
                files.extend(subdirs)
            files.sort()
            for name in files:
                for pattern in patterns:
                    if fnmatch.fnmatch(name, pattern):
                        yield os.path.join(path, name)
                        break
            if single_level:
                break

    def OpenProcessingFile(self, event): # wxGlade: MyFrame.<event_handler>
        """
        Prompt user to choose a dir.
        Display contents of dir in gui area in_input_dir
        update core and refresh content in transform queue
        """
        input_dir = ""
   
        dlg = wx.DirDialog(self, message="Choose a file")
        if self.Core.input_list_base:  #use cur selected dir
            dlg.SetPath(self.Core.input_list_base)
           
        if dlg.ShowModal() == wx.ID_OK:
            input_dir = dlg.GetPath() #unicode value returned
        dlg.Destroy()
          	
        if input_dir:
            self.text_list_in.Clear()              #clear the list displ
            self.textctrl_in.Clear()               #clear the ctrl displ
            self.textctrl_in.WriteText(input_dir)  #add new dir to ctrl displ
            self.Core.update_input_dir(input_dir)  #update core with new dir
            self.Core.update_preview_lists()       #update the preview lists
            #fill the interface widget text_list_in
            self.list_fill_text_ctrl(self.Core.in_input_dir, self.text_list_in)
            #clear/update gui items to show current values
            self.update_presentation()

        if self.Core.input_list_base and self.Core.output_list_base:
            self.btnStart.Enable()
            self.btnStop.Enable()


    def OpenOutputFile(self, event): # wxGlade: MyFrame.<event_handler>
        """
        Prompt user to choose a dir.
        Display contents of dir in gui area in_output_dir
        update core and refresh content in transform queue
        """
        input_dir = ""
        
        dlg = wx.DirDialog(self, message="Choose a file")
        if self.Core.output_list_base:
            dlg.SetPath(self.Core.output_list_base)
        
        if dlg.ShowModal() == wx.ID_OK:
            input_dir = dlg.GetPath()
        dlg.Destroy()
        
        if input_dir:
            self.text_list_out.Clear()
            self.textctrl_out.Clear()
            self.textctrl_out.WriteText(input_dir)
            self.Core.update_output_dir(input_dir)
            self.Core.update_preview_lists()
            #fill the interface widget text_list_out
            self.list_fill_text_ctrl(self.Core.in_output_dir, self.text_list_out)
            #clear/update gui items to show current values
            self.update_presentation()

        #self.button1.Hide()
        if self.Core.input_list_base and self.Core.output_list_base:
            self.btnStart.Enable()
            self.btnStop.Enable()


    def OnSize(self, event):
        if self.GetAutoLayout():
            self.Layout()
            self.prvw_img.Center()
        event.Skip()

    def list_box(self, event):
    
        show_preview = 0
    
        if self.list_box_queue.GetSelections():
            selected = self.list_box_queue.GetSelections()[0]
            img_path = self.Core.in_input_dir[selected]
            #print str(img_path)
            

            if os.path.exists(img_path) and image_verification(img_path):
            
                if (os.path.getsize(img_path) > FSIZE) and (self.proc_state == 1):      
                    bitmap = self.create_placeholder_image("+5 mb")
                    self.prvw_img.SetBitmap(bitmap)
                    self.OnSize(event)  
            
                else:
                  try:
                      #commented code uses PIL to preview, but its slower
                      #img = Image.open(img_path)
                      img = wx.Image(img_path, wx.BITMAP_TYPE_ANY)
                      w = self.prvw_pnl.GetSize()[0]
                      h = self.prvw_pnl.GetSize()[1]
                      #img = scale_image_fit_PIL(img, w, h)
                      img = rotate_image(img, self.Core.rotate)
                      img = scale_image_fit(img, w, h)
                      #image = wx.EmptyImage(img.size[0],img.size[1])
                      #image.SetData(img.convert("RGB").tostring())
                      #image.SetAlphaData(img.convert("RGBA").tostring()[3::4])
                      #img = wx.BitmapFromImage(image)             
                      img = img.ConvertToBitmap()

                      self.prvw_img.SetBitmap(wx.EmptyBitmap(img.GetSize()[0], img.GetSize()[1]))
                      self.prvw_img.SetBitmap(img)
                      self.OnSize(event)

                      self.notebook_1.SetPageText(1, self.Core.input_list_name[selected])
                      if not self.proc_state:
                          self.SetStatusText(str(self.Core.input_list_name[selected]) + self.Core.input_list_extn[selected])
                      
                  except IOError:
                      print "Preview Image Unavailable"
            else:
                self.SetStatusText("- !!!: " + self.Core.input_list_name[selected] + self.Core.input_list_extn[selected] + " --")
                bitmap = self.create_placeholder_image("- X -")
                self.prvw_img.SetBitmap(bitmap)
                self.OnSize(event)

    def cust_radio(self, event):
        cur_item = 0
        self.Core.cust_name = 1
        ## get currently selected item (if one exists)
        if self.list_box_queue.GetSelections(): # if something is selected
            cur_item = self.list_box_queue.GetSelections()[0]
            self.list_box_queue.Clear()
            self.Core.update_preview_lists()
            self.Core.update_listonly(self.list_box_queue)
            
            self.list_box_queue.SetSelection(cur_item)
            self.list_box(event)
        else:
            self.list_box_queue.Clear()
            self.Core.update_preview_lists()
            self.Core.update_listonly(self.list_box_queue)
            #self.update_presentation()

    def og_radio(self, event):
        cur_item = 0
        self.Core.cust_name = 0
        if self.list_box_queue.GetSelections(): # if something is selected
            cur_item = self.list_box_queue.GetSelections()[0]
            self.list_box_queue.Clear()
            self.Core.update_preview_lists()
            self.Core.update_listonly(self.list_box_queue)   
            
            self.list_box_queue.SetSelection(cur_item)
            self.list_box(event)   
      	
        else:
            self.list_box_queue.Clear()
            self.Core.update_preview_lists()
            self.Core.update_listonly(self.list_box_queue)
          	#list_box_fill(self.Core.in_input_dir, self.list_box_queue)
          	#self.update_presentation()

    def name_update(self, event):
      	self.Core.filename = self.text_ctrl_cust_name.GetValue()
      	self.list_box_queue.Clear()
      	self.Core.update_preview_lists()
      	self.Core.update_listonly(self.list_box_queue)
      	#self.update_presentation()

    def sequence_update(self, event):
      	#function prevents letters from getting input text_ctrl_cust_num
      	#a non-value/None is allowed 
      	number_string = self.text_ctrl_cust_num.GetValue()

      	if isstring(number_string) == "int_string":
        		##modify the list box at this stage
        		self.Core.sequence = int(number_string)
        		self.list_box_queue.Clear()
        		self.Core.sequence_length = len(number_string)	
        		self.Core.update_preview_lists()
        		self.Core.update_listonly(self.list_box_queue)
      	if isstring(number_string) == "abc_string":
        		self.text_ctrl_cust_num.Clear()
        		self.text_ctrl_cust_num.WriteText(remove_non_ints(number_string))
      	if isstring(number_string) == "non_string":
            self.Core.sequence = 0
            self.list_box_queue.Clear()
            self.Core.sequence_length = len(number_string)
            self.Core.update_preview_lists()
            self.Core.update_listonly(self.list_box_queue)

    def spin_iter(self, event):
    	self.Core.iteration = self.spin_ctrl_iter.GetValue()
    	self.list_box_queue.Clear()
    	self.Core.update_preview_lists()
    	self.Core.update_listonly(self.list_box_queue)
	
    def combo_format(self, event):
      #if combo_box_size is new then formatting must occur
      if self.Core.size != "---" and self.combo_box_format.GetValue() == "---":
          self.combo_box_format.SetSelection(1)
          self.combo_size(event)
          self.SetStatusText("- new size: formatting required --")
      if self.Core.rotate != "---" and self.combo_box_format.GetValue() == "---":
          self.combo_box_format.SetSelection(1)
          self.combo_size(event)
          self.SetStatusText("- rotation: formatting required --")
      self.Core.format = self.combo_box_format.GetValue()
      self.list_box_queue.Clear()
      self.Core.update_preview_lists()
      self.Core.update_listonly(self.list_box_queue)
      
    def combo_rotate(self, event):
      self.Core.rotate = self.combo_box_rotate.GetValue()
    
      if self.Core.rotate != "---" and self.combo_box_format.GetValue() == "---":
          self.combo_box_format.SetSelection(1)
          self.combo_format(event)
          self.SetStatusText("- rotation: formatting required --")
          
      
    def combo_size(self, event):
      self.Core.size = self.combo_box_size.GetValue()

      if self.Core.size == "thmb.":
          self.combo_box_measure.SetStringSelection("px")
          
      if self.Core.size == "medm.":
          self.combo_box_measure.SetStringSelection("px")
          
      if self.Core.size == "---":
          self.combo_box_measure.SetStringSelection("px")
          
      self.Core.update_size()
      self.spin_ctrl_width.SetValue(self.Core.width) 
      self.spin_ctrl_height.SetValue(self.Core.height)
      
      if self.Core.size != "---" and self.combo_box_format.GetValue() == "---":
          self.combo_box_format.SetSelection(1)
          self.combo_format(event)
          self.SetStatusText("- new size: formatting required --")

    	
    def update_presentation(self):
        """
        updates list_box, name_box, number_box items
        list_box   = self.list_box_queue
        name_box   = self.text_ctrl_cust_name
        number_box = self.text_ctrl_cust_num
        """
        if self.Core.in_input_dir: #user must have chosen an input dir
        
            #temp store cust_name, evt_text causing overwrite of Core.filename
            filename = self.Core.filename     
            self.text_ctrl_cust_name.Clear()
            self.text_ctrl_cust_name.WriteText(filename)

            #only update the number sequence if there's not sequence there
            if self.text_ctrl_cust_num.GetValue() == '':
                self.text_ctrl_cust_num.WriteText(str
                  (("%0" + str(self.Core.sequence_length) + "d") % (self.Core.sequence)))
        
            #combine image names with extensions
            combined_list = []
            for number, item in enumerate(self.Core.names_list):
                combined_list.append(
                  self.Core.names_list[number] + "." + self.Core.exten_list[number])

            self.list_box_queue.Clear()
            #display in the list_box
            for item in combined_list:
                self.list_box_queue.Append(os.path.basename(item))

    def list_fill_text_ctrl(self, focus_list, text_ctrl):
        """
        This function takes a list, itemizes it, and places
        the items into the given text ctrl
        """
        list_length = len(focus_list)
        x = 0
        while x < list_length:
            if x > 0:
                  text_ctrl.AppendText("\n")
            filename = submost_directory(focus_list[x])
            text_ctrl.AppendText(filename)
            x = x + 1
        text_ctrl.ShowPosition(0)

    def img_bck(self, event):
        """
        try moving back one image
        """
        cur_item = -1
        
        if self.list_box_queue.GetSelections(): # if something is selected
            cur_item += self.list_box_queue.GetSelections()[0]
            if cur_item >= 0:
                self.list_box_queue.SetSelection(cur_item)
                self.list_box(event)
                #self.img_stp(event)#on windows, image needs to be refreshed
            else:
                print "previous image unavailable"
                self.SetStatusText("- previous image unavailable --")
        else:
            self.SetStatusText("- image unavailable --")

    def img_stp(self, event):
        """
        try refresing the image
        """
        cur_item = 0
        
        if self.list_box_queue.GetSelections(): # if something is selected
            cur_item += self.list_box_queue.GetSelections()[0]
            self.list_box_queue.SetSelection(cur_item)
            self.list_box(event)
        else:
            self.SetStatusText("- image unavailable --")
            

    def img_fwd(self, event):
        """
        try moving forward one image
        """
        cur_item = 1
        
        if self.list_box_queue.GetSelections(): # if something is selected
            cur_item += self.list_box_queue.GetSelections()[0]
            if cur_item < len(self.Core.names_list):
                self.list_box_queue.SetSelection(cur_item)
                self.list_box(event)
                #self.img_stp(event)#on windows, image needs to be refreshed
            else:
                print "next image unavailable"
                self.SetStatusText("- next image unavailable --")
        else:
            self.SetStatusText("- image unavailable --")
        
        
    def spin_width(self, event):
        self.Core.width = self.spin_ctrl_width.GetValue()
        self.combo_box_size.SetSelection(3)
        self.combo_size(event)
        
    def spin_height(self, event):
        self.Core.height = self.spin_ctrl_height.GetValue()
        self.combo_box_size.SetSelection(3)
        self.combo_size(event)
        
    def combo_measure(self, event):
        self.Core.measurement = self.combo_box_measure.GetValue()
        self.combo_box_size.SetSelection(3)
     

    def update_input_list(self, event):
        "updates the list of input items"
        # only updates if new items found, adds new items to end of list to
        # maintain chronology

        content_new = self.Core.update_input_dir_chron()

        len_content = 0
        pos_content = 0
        cur_content = 0
        
        if content_new:
            self.Core.update_preview_lists()
            #fill the interface widget text_list_in
            self.text_list_in.Clear()
            self.list_fill_text_ctrl(self.Core.in_input_dir, self.text_list_in)
            #clear/update gui items to show current values
            #self.update_presentation()

            #append last 'content_new' items to the end of the text_ctrl 
            len_content = len(self.Core.input_list_name)
            pos_content = len_content - content_new

            while pos_content < len_content:
                cur_content = self.Core.names_list[pos_content]
                cur_content += "." + self.Core.exten_list[pos_content]
                #self.list_box_queue.Append(
                self.list_box_queue.Append(os.path.basename(cur_content))
                pos_content += 1

    def update_output_dir_list(self, event):
        "update text display for output directory"
        
        self.text_list_out.Clear()       #clear the list displ
        self.Core.update_output_dir(self.Core.output_list_base)
        #self.Core.update_preview_lists() #update preview lists
        #fill the interface widget text_list_out
        self.list_fill_text_ctrl(self.Core.in_output_dir, self.text_list_out)


    def update_status_bar(self, event):
        "update the status bar"
        if self.Core.content_new:
            old_file = self.Core.content_new[0]
            i = locate_matching_index(self.Core.content_new[0], self.Core.names_list)

            fname = self.Core.names_list[i]
            exten = self.Core.exten_list[i]
            self.SetStatusText("[...] " + fname + "." + exten + " processing...")
            self.list_box_queue.SetSelection(i)
            self.list_box(event)
            #print "old file: " + old_file
           
    def enable_render_controls(self, event):
        "enables controls in the render queue" 
        self.btn_in.Enable()
        self.textctrl_in.Enable()
        self.btn_out.Enable()
        self.textctrl_out.Enable()
        self.radio_btn_originals.Enable()
        self.radio_btn_customs.Enable()
        self.text_ctrl_cust_name.Enable()
        self.text_ctrl_cust_num.Enable()
        self.spin_ctrl_iter.Enable()
        self.combo_box_format.Enable()
        self.combo_box_rotate.Enable()
        self.combo_box_size.Enable()
        self.spin_ctrl_width.Enable()
        self.spin_ctrl_height.Enable()
        self.combo_box_measure.Enable()
        self.img_stp_btn.Enable()    
        self.img_bck_btn.Enable()
        self.img_fwd_btn.Enable()
               
    def disable_render_controls(self, event):
        "disables controls in the render queue" 
        self.btn_in.Disable()
        self.textctrl_in.Disable()
        self.btn_out.Disable()
        self.textctrl_out.Disable()
        self.radio_btn_originals.Disable()
        self.radio_btn_customs.Disable()
        self.text_ctrl_cust_name.Disable()
        self.text_ctrl_cust_num.Disable()
        self.spin_ctrl_iter.Disable()
        self.combo_box_format.Disable()
        self.combo_box_rotate.Disable()
        self.combo_box_size.Disable()
        self.spin_ctrl_width.Disable()
        self.spin_ctrl_height.Disable()
        self.combo_box_measure.Disable() 
        self.img_stp_btn.Disable()
        self.img_bck_btn.Disable()
        self.img_fwd_btn.Disable()       
        

    def start_proc(self, event):
      	"""
      	if an old thread is stopped, restart it,
      	else start a new thread
        """
        # two different timer, 
        # - fast timer; fire if new_files exist
        # - slow timer; fire if no new file exist
        if self.proc_state == 0:   # if first time through loop
            self.disable_render_controls(event)
            self.proc_state = 1
            self.Core.process_loop = True
                    
        if self.proc_state == 1: # if trying to process files
            if self.Core.process_loop: # if files queued to process
                if self.Core.thread_complete: # if thread not moving
                # fire quick threads until process_loop = 2 or 0
                  self.update_input_list(event) # ls imgs in input dir, prev
                  
                  if self.Core.content_new:
                    self.update_status_bar(event)
                    
                    if self.Core.update_listlog(self.list_box_queue): 
                      self.Core.content_fin.append(self.Core.content_new.pop(0))
                      self.update_status_bar(event)
                      self.update_output_dir_list(event) # ls imgs in output dir                        
                    else:
                      self.Core.activate_thread()

                  else:
                    self.Core.check_for_new()
                    if not self.Core.content_new:
                      self.Core.process_loop = False
 
                  self.make_timer("fast")
                    
            if not self.Core.process_loop: # if no files queued to process
                self.Core.process_loop = True  # set to check for new files
                if self.Core.thread_complete:
                #fire slow threads until process_remains 1 again
                  self.make_timer("slow")
                  self.SetStatusText("listening...")

        if self.proc_state == 4:
            self.make_timer("fast") # set the EVT_TIMER on start_btn
            #self.Core.activate_thread()
            self.proc_state = 1    

    def stop_proc(self, event):
        if self.proc_state == 1:
            self.proc_state = 3
        if self.proc_state == 3:
            #just kill the loop
            self.make_timer("off")
            self.proc_state += 1
            self.SetStatusText("state paused")
        elif self.proc_state == 4:
            self.proc_state = 0
            self.Core.reset()                     #set core to pre-proc state
            self.update_presentation()
            self.empty_preview_window(event)
            self.enable_render_controls(event)
            self.SetStatusText("state cleared")
        self.img_stp(event)
            
            
    def empty_preview_window(self, event):
        """
        """
        self.notebook_1.SetPageText(1, "Image")
        bitmap = self.create_placeholder_image("- X -")
        self.prvw_img.SetBitmap(bitmap)
        self.OnSize(event)   
        
            
    def make_timer(self, switch):
        if switch == "fast":
            if self.timer_loop != "fast":
                self.queue_timer_stop()
                self.timer_loop = "fast"
                self.queue_timer_fast()
        if switch == "slow":
            if self.timer_loop != "slow":
                self.queue_timer_stop()
                self.timer_loop = "slow"
                self.queue_timer_slow()
        if switch == "off":
            if self.timer_loop != "off":
                self.queue_timer_stop()
                self.timer_loop = "off"
        
        
