#contains common functions used by Stobo engine and gui
import wx #graphic lib
import Image #requires PIL
from math import floor
import os
import fnmatch
import shutil
import signal
import sys

def narrow_list_images(input_dir):
    """
    takes input directory and returns the sorted contents
    in alpha-numerical hierarchy
    """
    extensions = ''
    extensions += '*.jpg;*.jpeg;*.JPG;*.JPEG;'
    extensions += '*.png;*.PNG;'
    extensions += '*.gif;*.GIF;'
    extensions += '*.tif;*.tiff;*.TIF;*.TIFF;'
    extensions += '*.bmp;*.BMP'
    
    dir_list = list(all_files(input_dir, extensions))
    #remove images residing in subdirectories
    dir_list = isolate_list_directory(dir_list, input_dir)
    #dir_list = isolate_dir_contents(dir_list, input_dir)
    dir_list = sorted_copy(dir_list)
    #d looping timer threadir_list = submost_directory(dir_list)
    return dir_list
  
  
def dir_path_remove(list):
    """
    remove the directory path from a list of /full/path/files
    """
    new_list = []
    for item in list:
      #splitext creates two string item
      #0 = first item, and 1 = second item
      new_list.append(os.path.basename(item))
    return new_list
    
def all_files(root, patterns='*', single_level=False, yield_folders=False):
    """
    recipe from the Python Cookbook,
    walk a tree and filter on extension
    Q: why not glob.glob('Pictures/scan/*.*')?
    A: Can't specify directory
    """
    # Expand patterns from semicilon-separated string to list
    patterns = patterns.split(';')
    for path, subdirs, files in os.walk(root):
            if yield_folders:
                    files.extend(subdirs)
            files.sort()
            for name in files:
                    for pattern in patterns:
                            if fnmatch.fnmatch(name, pattern):
                                    yield os.path.join(path, name)
                                    break
            if single_level:
                    break
      
                 
def isolate_list_directory(focus_list, base_directory):
    """
    This function takes a list of directories and
    culls the list so directories past given depth are removed
    """
    list_length = len(focus_list)
    x = 0
    counter = 0
    new_list = []
    #get current item, remove path, match paths, if match keep
    while x < list_length:
      dir_name = focus_list[x]
      x = x + 1
      if base_directory == os.path.dirname(dir_name):
        new_list.append(dir_name)
        counter = counter + 1
    return new_list             
                    


def sorted_copy(alist):
    """
    modified from internet source by Alex Martelli
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52234
    """
    indices = map(generate_index, alist)
    decorated = zip(indices, alist)
    decorated.sort()
    return [ item for index, item in decorated ]
    
    
def generate_index(str):
    """
    Splits a string into alpha and numeric elements, which
    is used as an index for sorting"
    """
    index = []
    def _append(fragment, alist=index):
        if fragment.isdigit(): fragment = int(fragment)
        alist.append(fragment)

    # initialize loop
    prev_isdigit = str[0].isdigit()
    current_fragment = ''
    # group a string into digit and non-digit parts
    for char in str:
        curr_isdigit = char.isdigit()
        if curr_isdigit == prev_isdigit:
            current_fragment += char
        else:
            _append(current_fragment)
            current_fragment = char
            prev_isdigit = curr_isdigit
    _append(current_fragment)    
    return tuple(index)
    
    
def extensions_yield(list):
    """
    collects the extensions from list and 
    returns new list containing only the extensions
    """
    #declare a list
    new_list = []
    for item in list:
      #splitext creates two string item
      #0 = first item, and 1 = second item
      new_list.append(os.path.splitext(item)[1])
    return new_list

def extensions_remove(list):
    """
    collects the extensions from a list and
    returns new list containing no extensions
    """
    new_list = []
    for item in list:
        new_list.append(os.path.splitext(item)[0])
    return new_list


def append_list_to_list(add_list, orig_list):
    new_list = orig_list
    x = 0
    #for number, item in enumerate(add_list):
    #    new_list.append(self.add_list[number])
    for item in add_list:
        new_list.append(item)
    return new_list


def purge_matching_char(clean_me, purge_this):
    """
    returns the string 'clean_me' minus any occurrence
    of the char 'purge_this'
    """
    new_string = ""
    for char in clean_me:
        if char == purge_this:
            x = "do nothing"
        else:
            new_string += char
    return new_string
    

def purge_matching_elements(clean_me, purge_these):
    """
    returns a new list that contains only elements in clean_me,
    but not found in purge_these
    """
#    note: set does NOT preserver item order, making it unusable here...
#    print list(set(clean_me) - set(purge_these))
    ulist = []
    [ulist.append(x) for x in clean_me if x not in purge_these]
    return ulist
    
    
def submost_directory(dir_path):
    """
    returns the submost directory. 
    given a dir_path /home/duck/ will return duck.
    """
    while dir_path.endswith("/") or dir_path.endswith("\\"):
    #string[:]=fullslice and string[4:5]=slicetwixtchars4and5
        dir_path = dir_path[0:len(dir_path) - 1]
        # ^ cuts slice, does not include last char of string
        #print dir_path
    dir_path = os.path.basename(dir_path)
    return dir_path
    
def isstring(string):
  	"""
  	returns type of string
  	"""
  	try:
    		int(string)
  	except(ValueError):
    		x = "abc_string"
  	else:
    		x = "int_string"
  	
  	if string == "":
    		x = "non_string"
  	return x
	
def remove_non_ints(string):
    """ 
    remove abc chars from string 
    """
    new_string = ""
    char = ""
    x = len(string)
    y = 0
    while y < x:
        try:
            char = str(int(string[y]))
        except(ValueError):
            b = "do nothing"
        else:
            char = str(int(string[y]))
            new_string = new_string + char
        y = y + 1
    #if new_string == "":
    #	new_string = "0"
    return new_string
    
    
def locate_matching_index(string, list):
    """
    returns the list index of a 'list' item that
    has matching content with 'string'
    """
    x = ""
    for number, item in enumerate(list):
        if item == string:
            x = number
    return x


def scale_image_fit(img, fin_w, fin_h):
    """
    takes an Image() object and returns the image
    scaled to fit the dimensions of fin_w and fin_h
    """

    old_w = img.GetSize()[0]
    old_h = img.GetSize()[1]
    
    #convert to doubles
    old_w = old_w * 1.0000
    old_h = old_h * 1.0000
    fin_w = fin_w * 1.0000
    fin_h = fin_h * 1.0000
    
    new_h = fin_h
    new_w = floor(old_w * (fin_h / old_h)) 
    
    if (new_w > fin_w):
        new_w = fin_w
        new_h = floor(old_h * (fin_w / old_w))

    return img.Rescale(new_w, new_h, 100)
    
    
def scale_image_fit_PIL(img, fin_w, fin_h):
    """
    takes an Image() object and returns the image
    scaled to fit the dimensions of fin_w and fin_h
    """

    old_w, old_h = img.size
    
    #convert to doubles
    old_w = old_w * 1.0000
    old_h = old_h * 1.0000
    fin_w = fin_w * 1.0000
    fin_h = fin_h * 1.0000
    
    new_h = fin_h
    new_w = floor(old_w * (fin_h / old_h)) 
    
    if (new_h > fin_h) or (new_w > fin_w):
        new_w = fin_w
        new_h = floor(old_h * (fin_w / old_w))
    
    return img.resize((int(floor(new_w)), int(floor(new_h))), Image.ANTIALIAS)

def center_with_xy(img, fin_w, fin_h):
    """
    takes an Image() object and returns image
    coordinates that would center the image in
    a boundary of fin_w*fin_h.
    
    the img must _fit_ inside area fin_w*fin_h
    """
    
    img_w = img.GetSize()[0]
    img_h = img.GetSize()[1]
    coord = []
    
    if fin_w > img_w:
        coord.append((fin_w - img_w)/2)
    else:
        coord.append(0)
        
    if fin_h > img_h:
        coord.append((fin_h - img_h)/2)
    else:
        coord.append(0)
        
    return coord
        
def image_verification(img):
    """
    verifies that an image is, in fact, a valid image
    """
    is_image = 0
    
    if os.path.exists(img):
        try:
            verified = Image.open(img)
            verified.verify()
            is_image = 1
        except IOError:
            is_image = 0
        
    return is_image

def rotate_image(image, rotation):
    """
    rotates the image and returns it
    """
    new_image = image
    new_rotation = 0

    if rotation.isdigit():
        new_rotation = int(rotation)

    if new_rotation >= 90:
        new_image = new_image.Rotate90(clockwise=True)       

    if new_rotation >= 180:
        new_image = new_image.Rotate90(clockwise=True)

    if new_rotation >= 270:
        new_image = new_image.Rotate90(clockwise=True)

    return new_image















