import wx #graphic lib
import Image #requires PIL
import os
import fnmatch
import shutil
import threading
import signal
import time # for threading
import sys
import MainGui
from StoboCommon import *

class Engine(object):
    """
    Data and Data-Accessor functions used by the GUI
    """
    def __init__(self):
        "Data constructor for default values used by GUI"
        self.go_thread = 0         #will thread remain active. '1' for yes
        self.process_loop = False  #constantly queue dir, set '1' for true 
        
        self.names_list = []       #list to display in the queue
      	self.exten_list = []       #exts to display in the queue
      	# manipulation values
      	self.cust_name = 0         #if custom naming scheme, '1'
      	self.filename = "custom_"  #the selected img name
      	self.sequence = 0001       #first value of current sequence
      	self.sequence_length = 4   #number of 10's to appear in number seq
      	self.iteration = 1         #number to iterate by, ex: 3 -> 1,4,7,...
      	self.format = "---"        #img format, default '---' retain orig ext
      	self.rotate = "---"        #rotate img by this amount
      	self.size = "---"
      	self.width = 600           #width boundary for processed images
      	self.height = 600          #height boundary for processed images
      	self.measurement = "px"    #calculate width as inches, pixels, etc.
      	#list of image items in each directory
      	self.in_input_dir = ""     #images in input dir
      	self.in_output_dir = ""    #images in output dir 
      	#list of names for processed items
      	self.input_list_base = ""  #original img base path
      	self.input_list_name = ""  #original img base name (w/out path)
      	self.input_list_extn = ""  #original img ext
      	self.output_list_base = "" #new img base path
      	self.output_list_name = "" #new img base name (w/out path)
      	self.output_list_extn = "" #new img ext
      	#arrays of info on the processing status of the images
        self.content_new = []
        self.content_old = []
        self.content_fin = []
        #running cycle_engine in its own thread keeps stobo responsive
        self.thread = threading.Thread(target=self.cycle_engine) 
        #init with thread.start()
        self.thread = None
        self.thread_complete = True
        
        self.log = []

        """ setting input directory needs to activate new output file """
    def cycle_engine(self):
        """
        Process new files found
        Use check_for_new() before calling this function
        """
        self.thread_complete = False
        if self.process_loop:
            output_dir = self.output_list_base
            verify_img = 0
            #if its not necessary to process the images, rename them only
            formatting = not self.format == self.rotate == self.size == "---"
        
            if self.content_new and formatting: #proc item in content_new
                old_file = self.content_new[0]
                i = locate_matching_index(self.content_new[0], self.names_list)
                filename = self.names_list[i]
                extension = self.exten_list[i]

                new_file = os.path.join(output_dir,filename) + "." + extension
                print "[...] " + old_file + " processing..."
                
                verify_img = image_verification(self.in_input_dir[i])
                if verify_img:
                    try:
                        im = Image.open(self.in_input_dir[i])
                        if self.size != "---":
                            im = scale_image_fit_PIL(im, self.width, self.height)
                        if self.rotate != "---":
                            im = im.rotate(float(self.rotate))
                        im.save(new_file, extension)
                        self.log.append("[...] ")
                    except IOError:
                	      print "[!!!] " + old_file
                	      self.log.append("[!!!] ")
                else:
                    print "[!!!] cannot process " + old_file
                    self.log.append("[!!!] ")
                
            elif self.content_new and not formatting:
                old_file = self.content_new[0]
                i = locate_matching_index(self.content_new[0], self.names_list)
                filename = self.names_list[i]
                extension = self.exten_list[i]
                
                new_file = os.path.join(output_dir,filename) + "." + extension
                print "[...] " + old_file + " processing..."

                verify_img = image_verification(self.in_input_dir[i])
                if verify_img:
                    try:
                        shutil.copyfile(self.in_input_dir[i], new_file)
                        self.log.append("[...] ")
                    except IOError:
                	      print "[!!!] " + old_file
                	      self.log.append("[!!!] ")
                else:
                    print "[!!!] cannot process " + old_file
                    self.log.append("[!!!] ")               
            else:
              self.process_loop = False
        self.check_for_new() # background check_for_new here
        self.thread_complete = True
        return str("end thread") 


    def check_for_new(self):
        """
        checks for the appearance of new files in the input_dir
        """
    		#purge matches w/content_old, a list of images already discovered
        proc_queue = purge_matching_elements(self.names_list, self.content_old)

        self.content_new.extend(proc_queue)
        self.content_old.extend(proc_queue)
    	  
    	  #purge matches w/content_fin, a list of images already processed
        self.content_new = purge_matching_elements(self.content_new, self.content_fin)


    def activate_thread(self):
        """
        run cycle_engine() in a new thread
        """
        self.thread_complete = False
        
        if self.thread:
            self.thread.join()
        self.thread = None
        self.thread = threading.Thread(target=self.cycle_engine)
        self.thread.start() #init the thread
        #print "activated thread"


    def update_input_dir(self, new_dir):
        """
        update the input list to use new_dir
        """
        self.input_list_base = new_dir
        
        self.in_input_dir = narrow_list_images(new_dir)                 # path
        self.input_list_extn = extensions_yield(self.in_input_dir)      # ext
        self.input_list_name = dir_path_remove(self.in_input_dir) 
        self.input_list_name = extensions_remove(self.input_list_name)  # name

    
    def update_input_dir_chron(self):
        """
        update the input list to use new dir, chronologically
        """
        x = 0
        new_input_dir = []
        old_input_dir = self.in_input_dir

        #get images from input dir
        new_input_dir = narrow_list_images(self.input_list_base)
        #remove images found in old_input_dir from new_input_dir
        new_input_dir = purge_matching_elements(new_input_dir, old_input_dir)
        if new_input_dir: 
            self.in_input_dir = append_list_to_list(new_input_dir, old_input_dir)
            
            self.input_list_extn = extensions_yield(self.in_input_dir) # ext
            self.input_list_name = dir_path_remove(self.in_input_dir) 
            self.input_list_name = extensions_remove(self.input_list_name)# name
            x = len(new_input_dir)

        return x # return number of items in list new_input_dir
    
      	
    def update_output_dir(self, new_dir):
        """
        update the input list to use new_dir
        """
        self.output_list_base = new_dir
        self.in_output_dir = narrow_list_images(new_dir)  # path
        self.output_list_extn = extensions_yield(self.in_output_dir) # ext
        self.output_list_name = dir_path_remove(self.in_output_dir)
        self.output_list_name = extensions_remove(self.output_list_name)# name
        
      	
    def update_preview_lists(self):
    
        """
        combine names, numbers and extensions and show full
        img names as preview in the list box
        """
        names_list = []
        exten_list = []
        final_list = []
        ##list of names (original)
        if self.cust_name == 0:
            for number, item in enumerate(self.in_input_dir):
                names_list.append(self.input_list_name[number])
        ##list of names (cust_name)
        if self.cust_name == 1:
            num = self.sequence
            for number, item in enumerate(self.in_input_dir):
                names_list.append(self.filename + str(
                  ("%0" + str(self.sequence_length) + "d") % (num))) 
                num += self.iteration

        ##list of extensions (original), .lower() activates lower casing...
        if self.format == "---":
            for number, item in enumerate(self.in_input_dir):
                exten_list.append(purge_matching_char(self.input_list_extn[number], ".").lower()) 
        ##list of extensions (custon)
        else:
            for item in (self.in_input_dir):
                exten_list.append(self.format) #"." + Tranval.format)

        self.names_list = names_list
        self.exten_list = exten_list


    def update_listonly(self, list_box):
        """
        some items, like cust_name box have text evt-handlers
        update list box only and avoid any evt handler loop
        """
        combined_list = []
        for number, item in enumerate(self.names_list):
            combined_list.append(
                self.names_list[number] + "." + self.exten_list[number])
                    
        list_box.Clear()

        for item in combined_list:
		        list_box.Append(os.path.basename(item))
        #list_box_fill(combined_list, list_box)
        
        
    def update_listlog(self, list_box):
        """
        if selected item in list_box
        get currently selected item in list_box
        get the index of that item
        if the index exitss in log, change item to include_log
     
        """
        cur_item = 0
        x = 0
        if list_box.GetSelections():                # if something is selected
            cur_item += list_box.GetSelections()[0] # get selected item
            cur_str = list_box.GetString(cur_item)  # get the item string
            if cur_item <= len(self.log): 
              try:
                log = self.log[cur_item]
                list_box.SetString(cur_item, log + cur_str)
                x=1
              except IndexError, e:
                pass
        return x  
        
        
    def update_size(self):
      """
      ---   -> nothing
      thmb. -> 400x400 px
      medm. -> 1000x1000 px
      cust. -> nothing
      """
      if self.size == "thmb.":
          self.width = 400
          self.height = 400
      if self.size == "medm.":
          self.width = 1000
          self.height = 1000
      if self.size == "---":
          self.width = 0
          self.height = 0
          
    def reset(self):
        "reset these classes' core lists to empty pre-process state"
        self.thread.join()
        self.thread = None
        
        self.content_new = []
        self.content_old = []
        self.content_fin = []
        
        self.log = []
        
        self.update_input_dir(self.input_list_base)
        self.update_preview_lists()
        
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      


